@@ -1,5 +1,5 @@
 /* Extended regular expression matching and search library.
-   Copyright (C) 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2003 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
    Contributed by Isamu Hasegawa <isamu@yamato.ibm.com>.
 
@@ -33,7 +33,7 @@
 #include <string.h>
 
 #if defined HAVE_LOCALE_H || defined _LIBC
-#include <locale.h>
+# include <locale.h>
 #endif
 #if defined HAVE_WCHAR_H || defined _LIBC
 # include <wchar.h>
@@ -74,11 +74,11 @@
 # define gettext_noop(String) String
 #endif
 
-#if (defined (MB_CUR_MAX) && HAVE_WCSCOLL && HAVE_WCTYPE_H && HAVE_WCHAR_H && HAVE_WCRTOMB && HAVE_MBRTOWC) || _LIBC
-#define RE_ENABLE_I18N
+#if (defined MB_CUR_MAX && HAVE_LOCALE_H && HAVE_WCTYPE_H && HAVE_WCHAR_H && HAVE_WCRTOMB && HAVE_MBRTOWC && HAVE_WCSCOLL) || _LIBC
+# define RE_ENABLE_I18N
 #endif
 
-#if _LIBC || __GNUC__ >= 3
+#if __GNUC__ >= 3
 # define BE(expr, val) __builtin_expect (expr, val)
 #else
 # define BE(expr, val) (expr)
@@ -101,10 +101,17 @@
 # define __wctype wctype
 # define __iswctype iswctype
 # define __btowc btowc
-# define __mempcpy memcpy
+# define __mempcpy mempcpy
+# define __wcrtomb wcrtomb
 # define attribute_hidden
 #endif /* not _LIBC */
 
+#ifdef __GNUC__
+# define __attribute __attribute__
+#else
+# define __attribute
+#endif
+
 extern const char __re_error_msgid[] attribute_hidden;
 extern const size_t __re_error_msgid_idx[] attribute_hidden;
 
@@ -175,6 +182,8 @@ typedef enum
   OP_CLOSE_CHAR_CLASS,
   OP_WORD,
   OP_NOTWORD,
+  OP_SPACE,
+  OP_NOTSPACE,
   BACK_SLASH,
 
   /* Tree type, these are used only by tree. */
@@ -465,6 +474,8 @@ struct re_state_table_entry
   re_dfastate_t **array;
 };
 
+/* Array type used in re_sub_match_last_t and re_sub_match_top_t.  */
+
 typedef struct
 {
   int next_idx;
@@ -472,24 +483,28 @@ typedef struct
   re_dfastate_t **array;
 } state_array_t;
 
+/* Store information about the node NODE whose type is OP_CLOSE_SUBEXP.  */
+
 typedef struct
 {
   int node;
-  int str_idx;
+  int str_idx; /* The position NODE match at.  */
   state_array_t path;
-  re_node_set *limits;
 } re_sub_match_last_t;
 
+/* Store information about the node NODE whose type is OP_OPEN_SUBEXP.
+   And information about the node, whose type is OP_CLOSE_SUBEXP,
+   corresponding to NODE is stored in LASTS.  */
+
 typedef struct
 {
   int str_idx;
   int node;
   int next_last_offset;
-  int alasts;
-  int nlasts;
-  re_sub_match_last_t **lasts;
   state_array_t *path;
-  re_node_set *limits;
+  int alasts; /* Allocation size of LASTS.  */
+  int nlasts; /* The number of LASTS.  */
+  re_sub_match_last_t **lasts;
 } re_sub_match_top_t;
 
 struct re_backref_cache_entry
@@ -499,8 +514,6 @@ struct re_backref_cache_entry
   int subexp_from;
   int subexp_to;
   int flag;
-  re_sub_match_top_t *top;
-  re_sub_match_last_t *last;
 };
 
 typedef struct
@@ -568,6 +581,7 @@ struct re_dfa_t
   int nodes_len;
   bin_tree_t *str_tree;
   int *nexts;
+  int *org_indices;
   re_node_set *edests;
   re_node_set *eclosures;
   re_node_set *inveclosures;
@@ -580,13 +594,15 @@ struct re_dfa_t
   int states_alloc;
   int init_node;
   int nbackref; /* The number of backreference in this dfa.  */
-  /* If this dfa has "multibyte node", which is a backreference or
-     a node which can accept multibyte character or multi character
-     collating element.  */
+  /* Bitmap expressing which backreference is used.  */
+  unsigned int used_bkref_map;
 #ifdef DEBUG
   char* re_str;
 #endif
   unsigned int has_plural_match : 1;
+  /* If this dfa has "multibyte node", which is a backreference or
+     a node which can accept multibyte character or multi character
+     collating element.  */
   unsigned int has_mb_node : 1;
 };
 typedef struct re_dfa_t re_dfa_t;
