@@ -80,7 +80,6 @@ extern int errno;
 
 #include <sys/stat.h>
 
-#include "basicdefs.h"
 #include "utils.h"
 #include "sed.h"
 
@@ -538,6 +537,9 @@ open_next_file(name, input)
       output_fd = mkstemp (input->out_file_name);
       free (tmpdir);
 
+      if (output_fd == -1)
+        panic(_("Couldn't open temporary file %s: %s"), input->out_file_name, strerror(errno));
+
 #ifdef HAVE_FCHMOD
       {
 	struct stat st;
@@ -547,6 +549,8 @@ open_next_file(name, input)
 #endif
 
       output_file = fdopen (output_fd, "w");
+      if (!output_file)
+        panic(_("Couldn't open temporary file %s: %s"), input->out_file_name, strerror(errno));
     }
   else
     output_file = stdout;
@@ -565,19 +569,21 @@ closedown(input)
   if (input->fp != stdin) /* stdin can be reused on tty and tape devices */
     ck_fclose(input->fp);
 
-      if (in_place_extension && output_file != NULL)
-	{
-	  if (strcmp(in_place_extension, "*") != 0)
-	    {
-	      char *backup_file_name = get_backup_file_name(input->in_file_name);
-	      rename (input->in_file_name, backup_file_name);
-	      free (backup_file_name);
-	    }
+  if (in_place_extension && output_file != NULL)
+    {
+      int fd = fileno (output_file);
+      if (strcmp(in_place_extension, "*") != 0)
+        {
+          char *backup_file_name = get_backup_file_name(input->in_file_name);
+          rename (input->in_file_name, backup_file_name);
+          free (backup_file_name);
+        }
 
-	  ck_fclose (output_file);
-	  rename (input->out_file_name, input->in_file_name);
-	  free (input->out_file_name);
-	}
+      ck_fclose (output_file);
+      close (fd);
+      rename (input->out_file_name, input->in_file_name);
+      free (input->out_file_name);
+    }
 
   input->fp = NULL;
 
@@ -1127,8 +1133,6 @@ execute_program(vec, input)
 		 but it seems to be expected (and make sense). */
 	      /* Fall Through */
 	    case 'd':
-	      line.length = 0;
-	      line.chomped = FALSE;
 	      return -1;
 
 	    case 'D':
@@ -1264,13 +1268,12 @@ execute_program(vec, input)
 	      }
 	      break;
 
-	    case 'Q':
-	      line.length = 0;
-	      line.chomped = FALSE;
-	      /* Fall through */
+            case 'q':
+              if (!no_default_output)
+                output_line(line.active, line.length, line.chomped, output_file);
 
-	    case 'q':
-	      return cur_cmd->x.int_arg == -1 ? 0 : cur_cmd->x.int_arg;
+            case 'Q':
+              return cur_cmd->x.int_arg == -1 ? 0 : cur_cmd->x.int_arg;
 
 	    case 'r':
 	      if (cur_cmd->x.fname)
@@ -1481,6 +1484,9 @@ execute_program(vec, input)
       /* this is buried down here so that a "continue" statement can skip it */
       ++cur_cmd;
     }
+
+    if (!no_default_output)
+      output_line(line.active, line.length, line.chomped, output_file);
     return -1;
 }
 
@@ -1516,8 +1522,6 @@ process_files(the_program, argv)
   while (read_pattern_space(&input, the_program, FALSE))
     {
       status = execute_program(the_program, &input);
-      if (!no_default_output)
-	output_line(line.active, line.length, line.chomped, output_file);
       if (status == -1)
 	status = EXIT_SUCCESS;
       else
